/*********************************************************************
 * Software License Agreement (BSD License)
 *
 *  Copyright (c) 2022, Kassow Robots
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the Kassow Robots nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *********************************************************************/

#ifndef KR2_KORD_SERVER_H
#define KR2_KORD_SERVER_H

#include <vector>
#include <string>
#include <kr2/utils/commands_monitor.h>

namespace kr2 {
namespace kord {

enum class EConnType{
    KORD_TCP_CONN = 1,
    KORD_UDP_CONN = 2
};

typedef timespec tick_t;
typedef unsigned long KORDItemId;

class RobotControl;
class RobotResponseItem;
class RobotModel;
class RobotController;

struct CommandStatus;
template<typename T>
struct StatusBuffer;

class KORDServer {
    public:
        KORDServer() = delete;

        KORDServer(int port, int session_id, EConnType conn_type, unsigned int tx_freq);

        KORDServer(const KORDServer&) = delete;

        ~KORDServer();

        //! Waits for the remote host connection with given timeout
        //! Starts the transmission of the status frame at a given rate
        //! Starts capture of the data from the interface
        bool open(double timeout);
        
        //! If a connection is already active return true
        bool isActive() const;

        //! Finish transmission and close the connection
        bool close();

        //! Fetch recent request ids
        std::vector<KORDItemId> fetchItemsIDs() const;

        //! Fetch latest capture data
        bool fetchItem(RobotControl*, tick_t recent_timeout, CommandsMonitor&, const RobotController &);

        //! Update ARM status frame with recent data
        void updateStatus(const RobotModel &, const RobotController &, StatusBuffer<CommandStatus> &);

        //! Update Content frame with provided data
        bool addResponse(RobotResponseItem*);

        //! Send arm status frame
        //! Send content frame with reponse items (if available in the response buffer)
        //! Clear both status and content frames
        bool dispatch(int64_t& sent_status_tx_stamp, CommandsMonitor&);

    private:
        class KORDServerImpl;
        KORDServerImpl *impl_;

};

}  // kord
}  // kr2

#endif  // KR2_KORD_SERVER_H
